##### demonstration of extracellular stimulation of a model cell

from neuron import h, gui

##### model cell

h.load_file("sphere.hoc")

somaR = h.somaR # convenient Python aliases for section names
somaL = h.somaL

NSEG = 11 # used by discretize() below

for sec in h.allsec():
  sec.insert('extracellular')
  sec.insert('xtra')

##### spatial discretization and extracellular stimulation

# assumes cell is centered on 0,0,0
# extracellular potential is 0 at 0,0,0 (convenient for graphs)
# potential gradient is linear and parallel to x axis
# sec   section to be processed
# ns    nseg
# dedx  gradient in v/m
# (gradient is numerically equal to the value of is)
def discretize(sec, ns, dedx):
  sec.nseg = ns
  # now that internal nodes exist,
  # link extracellular and xtra
  for seg in sec:
    seg.xtra._ref_im = seg._ref_i_membrane
    seg.xtra._ref_ex = seg._ref_e_extracellular
  # relationship between location and e_extracellular
  # don't use xtra's er for anything
  for seg in sec:
    xloc = (sec.diam3d(0)/2)*seg.x # distance of seg's node from 0 end
    if (sec.x3d(1) < 0): xloc = -xloc # left hemisphere
    seg.xtra.rx = 1e-9 * dedx * xloc

for sec in h.allsec():
  discretize(sec, NSEG, 1)

##### user interface

h.load_file("rig.ses") # RunControl, Movie Run, 
  # PPM as IClamp at somaL(0)
  # plot of v vs. t for 0, 0.5, and 1 locations on somaR and somaL
  # v vs. position from somaL(1) to somaR(1)
  # e_extracellular vs. position from somaL(1) to somaR(1)

##### extracellular stimulus

gvec = h.Vector(6) # gradient--0 when field off, nonzero when on
tvec = h.Vector(6)
gvec.play(h._ref_is_xtra, tvec, 1) # interpolated play

# ton, dur are stimulus start time and duration
# grad is stimulus-elicited potential gradient in kv/m
#   which is numerically identical to potential gradient in mv/um

def makestim(ton, dur, grad):
  tvec.from_python([0, ton, ton,      ton+dur,  ton+dur, ton+dur+1])
  # 1e3 ensures that grad = 1 produces potential gradient 1 mv/um
  gvec.from_python([0, 0,   grad*1e3, grad*1e3, 0,       0])

makestim(1, 100, 1.57263*2) # gradient just suprathreshold
                            #   for 5 um diam soma

# report the actual extracellular gradient

def gradient():
  # dv between last and first internal nodes
  dv = somaR(0.999).e_extracellular - somaR(0.001).e_extracellular
  h.distance(0, 1/2/somaR.nseg) # origin: first internal node
  dx = h.distance(1 - 1/2/somaR.nseg) # distance to last internal node
  return dv/dx

def grad():
  print("extracellular potential gradient", gradient(), "mv/um")
